<p class="logo">
  <a href="{{ pathto(master_doc) }}">
    <img class="logo" src="{{ pathto('_static/logo.svg', 1) }}" alt="Logo"/>
  </a>
</p>
<p>&nbsp;</p>
{% if theme_description %}
<p class="blurb">{{ theme_description }}</p>
{% endif %}
