{% extends "email_layout.html" %}

{% block content %}
<p>Hi there,</p>
<p>Someone created for you a {{app_name}} user account with username {{user.username}}.</p>
<p>Please go to <a href="{{url}}">{{url}}</a> and complete your profile.</p>
<p>This link expires after {{expires}} hours.</p>
<p>Please ignore this email if you think your {{app_name}} account was created in error.</p>
{% endblock %}
