{% extends "email_layout.html" %}

{% block content %}
<p>The following original item has been modified by {{username}}.</p>
<p>Please make sure to update the corresponding translated item accordingly. <a href="{{link}}">{{headline}}</a></p>
{% endblock %}
