# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


LINKED_IN_PACKAGES = "linked_in_packages"
PACKAGE = "package"
PACKAGE_TYPE = "package_type"
REFS = "refs"
RESIDREF = "residRef"
ID_REF = "idRef"
ROLE = "role"
GROUPS = "groups"
MAIN_GROUP = "main"
MAIN_ROLE = "grpRole:main"
ROOT_GROUP = "root"
ROOT_ROLE = "grpRole:NEP"
GROUP_ID = "id"
